@@ -22,17 +22,13 @@ fdiff.default <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, log
   .Call(Cpp_fdiffgrowth,x,n,diff,fill,g[[1L]],g[[2L]],g[[3L]],G_t(t),1L+log,rho,stubs,1)
 }
 
-fdiff.pseries <- function(x, n = 1, diff = 1, fill = NA, log = FALSE, rho = 1, stubs = TRUE, ...) {
+fdiff.pseries <- function(x, n = 1, diff = 1, fill = NA, log = FALSE, rho = 1, stubs = length(n) + length(diff) > 2L, shift = "time", ...) {
   if(!missing(...)) if(checkld(...)) log <- list(...)[["logdiff"]] else unused_arg_action(match.call(), ...)
+  index <- uncl2pix(x)
   if(log) x <- baselog(x)
-  index <- unclass(getpix(attr(x, "index")))
-  if(length(index) > 2L) index <- list(finteraction(index[-length(index)]), index[[length(index)]])
   g <- index[[1L]]
-  t <- index[[2L]]
-  tlev <- attr(t, "levels")
-  oldopts <- options(warn = -1L)
-  on.exit(options(oldopts))
-  if(is.finite(as.integer(tlev[1L]))) t <- as.integer(tlev)[t]
+  t <- switch(shift, time = index[[2L]], row = NULL, stop("'shift' must be either 'time' or 'row'"))
+  if(length(t) && !inherits(x, "indexed_series")) t <- plm_check_time(t)
   if(is.matrix(x))
     .Call(Cpp_fdiffgrowthm,x,n,diff,fill,fnlevels(g),g,NULL,t,1L+log,rho,stubs,1) else
       .Call(Cpp_fdiffgrowth,x,n,diff,fill,fnlevels(g),g,NULL,t,1L+log,rho,stubs,1)
@@ -49,18 +45,15 @@ fdiff.matrix <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, log
 fdiff.grouped_df <- function(x, n = 1, diff = 1, t = NULL, fill = NA, log = FALSE, rho = 1, stubs = length(n) + length(diff) > 2L, keep.ids = TRUE, ...) {
   if(!missing(...)) if(checkld(...)) log <- list(...)[["logdiff"]] else unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x, call = FALSE)
-  tsym <- all.vars(substitute(t))
+  tsym <- substitute(t)
   nam <- attr(x, "names")
   gn <- which(nam %in% g[[5L]])
-  if(length(tsym) && !anyNA(tn <- match(tsym, nam))) {
-    if(length(tn) == 1L) {
-      if(any(gn == tn)) stop("timevar coincides with grouping variables!")
-      t <- .subset2(x, tn)
-    } else {
-      if(any(gn %in% tn)) stop("timevar coincides with grouping variables!")
-      t <- .subset(x, tn)
+  if(!is.null(tsym)) {
+    t <- eval(tsym, x, parent.frame())
+    if(!anyNA(tn <- match(all.vars(tsym), nam))) {
+      gn <- c(gn, tn)
+      if(anyDuplicated.default(gn)) stop("timevar coincides with grouping variables!")
     }
-    gn <- c(gn, tn)
   }
   cld <- function(x) if(log) fdapply(x, baselog) else x
   if(length(gn)) {
@@ -81,20 +74,15 @@ fdiff.data.frame <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA,
   .Call(Cpp_fdiffgrowthl,x,n,diff,fill,g[[1L]],g[[2L]],g[[3L]],G_t(t),1L+log,rho,stubs,1)
 }
 
-fdiff.list <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, log = FALSE, rho = 1, stubs = length(n) + length(diff) > 2L, ...)
-  fdiff.data.frame(x, n, diff, g, t, fill, log, rho, stubs, ...)
+fdiff.list <- function(x, ...) fdiff.data.frame(x, ...)
 
-fdiff.pdata.frame <- function(x, n = 1, diff = 1, fill = NA, log = FALSE, rho = 1, stubs = length(n) + length(diff) > 2L, ...) {
+fdiff.pdata.frame <- function(x, n = 1, diff = 1, fill = NA, log = FALSE, rho = 1, stubs = length(n) + length(diff) > 2L, shift = "time", ...) {
   if(!missing(...)) if(checkld(...)) log <- list(...)[["logdiff"]] else unused_arg_action(match.call(), ...)
+  index <- uncl2pix(x)
   if(log) x <- fdapply(x, baselog)
-  index <- unclass(getpix(attr(x, "index")))
-  if(length(index) > 2L) index <- list(finteraction(index[-length(index)]), index[[length(index)]])
   g <- index[[1L]]
-  t <- index[[2L]]
-  tlev <- attr(t, "levels")
-  oldopts <- options(warn = -1L)
-  on.exit(options(oldopts))
-  if(is.finite(as.integer(tlev[1L]))) t <- as.integer(tlev)[t]
+  t <- switch(shift, time = index[[2L]], row = NULL, stop("'shift' must be either 'time' or 'row'"))
+  if(length(t) && !inherits(x, "indexed_frame")) t <- plm_check_time(t)
   .Call(Cpp_fdiffgrowthl,x,n,diff,fill,fnlevels(g),g,NULL,t,1L+log,rho,stubs,1)
 }
 
@@ -112,17 +100,13 @@ fgrowth.default <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, l
   .Call(Cpp_fdiffgrowth,x,n,diff,fill,g[[1L]],g[[2L]],g[[3L]],G_t(t),4L-logdiff,scale,stubs,power)
 }
 
-fgrowth.pseries <- function(x, n = 1, diff = 1, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, ...) {
+fgrowth.pseries <- function(x, n = 1, diff = 1, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = length(n) + length(diff) > 2L, shift = "time", ...) {
   if(!missing(...)) unused_arg_action(match.call(), ...)
+  index <- uncl2pix(x)
   if(logdiff) x <- if(scale == 1) baselog(x) else scale * baselog(x)
-  index <- unclass(getpix(attr(x, "index")))
-  if(length(index) > 2L) index <- list(finteraction(index[-length(index)]), index[[length(index)]])
   g <- index[[1L]]
-  t <- index[[2L]]
-  tlev <- attr(t, "levels")
-  oldopts <- options(warn = -1L)
-  on.exit(options(oldopts))
-  if(is.finite(as.integer(tlev[1L]))) t <- as.integer(tlev)[t]
+  t <- switch(shift, time = index[[2L]], row = NULL, stop("'shift' must be either 'time' or 'row'"))
+  if(length(t) && !inherits(x, "indexed_series")) t <- plm_check_time(t)
   if(is.matrix(x))
     .Call(Cpp_fdiffgrowthm,x,n,diff,fill,fnlevels(g),g,NULL,t,4L-logdiff,scale,stubs,power) else
       .Call(Cpp_fdiffgrowth,x,n,diff,fill,fnlevels(g),g,NULL,t,4L-logdiff,scale,stubs,power)
@@ -139,18 +123,15 @@ fgrowth.matrix <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, lo
 fgrowth.grouped_df <- function(x, n = 1, diff = 1, t = NULL, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = length(n) + length(diff) > 2L, keep.ids = TRUE, ...) {
   if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x, call = FALSE)
-  tsym <- all.vars(substitute(t))
+  tsym <- substitute(t)
   nam <- attr(x, "names")
   gn <- which(nam %in% g[[5L]])
-  if(length(tsym) && !anyNA(tn <- match(tsym, nam))) {
-    if(length(tn) == 1L) {
-      if(any(gn == tn)) stop("timevar coincides with grouping variables!")
-      t <- .subset2(x, tn)
-    } else {
-      if(any(gn %in% tn)) stop("timevar coincides with grouping variables!")
-      t <- .subset(x, tn)
+  if(!is.null(tsym)) {
+    t <- eval(tsym, x, parent.frame())
+    if(!anyNA(tn <- match(all.vars(tsym), nam))) {
+      gn <- c(gn, tn)
+      if(anyDuplicated.default(gn)) stop("timevar coincides with grouping variables!")
     }
-    gn <- c(gn, tn)
   }
   cld <- function(x) if(!logdiff) x else if(scale != 1) fdapply(x, function(y) scale * baselog(y)) else fdapply(x, baselog)
   if(length(gn)) {
@@ -171,20 +152,15 @@ fgrowth.data.frame <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA
   .Call(Cpp_fdiffgrowthl,x,n,diff,fill,g[[1L]],g[[2L]],g[[3L]],G_t(t),4L-logdiff,scale,stubs,power)
 }
 
-fgrowth.list <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = length(n) + length(diff) > 2L, ...)
-  fgrowth.data.frame(x, n, diff, g, t, fill, logdiff, scale, power, stubs, ...)
+fgrowth.list <- function(x, ...) fgrowth.data.frame(x, ...)
 
-fgrowth.pdata.frame <- function(x, n = 1, diff = 1, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = length(n) + length(diff) > 2L, ...) {
+fgrowth.pdata.frame <- function(x, n = 1, diff = 1, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = length(n) + length(diff) > 2L, shift = "time", ...) {
   if(!missing(...)) unused_arg_action(match.call(), ...)
+  index <- uncl2pix(x)
   if(logdiff) x <- if(scale == 1) fdapply(x, baselog) else fdapply(x, function(y) scale * baselog(y))
-  index <- unclass(getpix(attr(x, "index")))
-  if(length(index) > 2L) index <- list(finteraction(index[-length(index)]), index[[length(index)]])
   g <- index[[1L]]
-  t <- index[[2L]]
-  tlev <- attr(t, "levels")
-  oldopts <- options(warn = -1L)
-  on.exit(options(oldopts))
-  if(is.finite(as.integer(tlev[1L]))) t <- as.integer(tlev)[t]
+  t <- switch(shift, time = index[[2L]], row = NULL, stop("'shift' must be either 'time' or 'row'"))
+  if(length(t) && !inherits(x, "indexed_frame")) t <- plm_check_time(t)
   .Call(Cpp_fdiffgrowthl,x,n,diff,fill,fnlevels(g),g,NULL,t,4L-logdiff,scale,stubs,power)
 }
 
@@ -208,7 +184,7 @@ DG_data_frame_template <- function(x, n = 1, diff = 1, by = NULL, t = NULL, cols
         gn <- ckmatch(all.vars(by[[3L]]), nam)
       } else {
         gn <- ckmatch(all.vars(by), nam)
-        cols <- if(is.null(cols)) seq_along(x)[-gn] else cols2int(cols, x, nam)
+        cols <- cols2intrmgn(gn, cols, x)
       }
       by <- G_guo(if(length(gn) == 1L) x[[gn]] else x[gn])
       if(!keep.ids) gn <- NULL
@@ -221,7 +197,7 @@ DG_data_frame_template <- function(x, n = 1, diff = 1, by = NULL, t = NULL, cols
     if(is.call(t)) {
       tn <- ckmatch(all.vars(t), nam)
       t1 <- length(tn) == 1L
-      t <- if(t1) x[[tn]] else x[tn]
+      t <- eval(if(t1) t[[2L]] else attr(terms.formula(t), "variables"), x, attr(t, ".Environment")) # if(t1) x[[tn]] else x[tn]
       cols <- if(is.null(cols)) seq_along(x)[-tn] else if(t1) cols[cols != tn] else fsetdiff(cols, tn)
       if(keep.ids) gn <- c(gn, tn)
     }
@@ -244,31 +220,33 @@ DG_data_frame_template <- function(x, n = 1, diff = 1, by = NULL, t = NULL, cols
   .Call(Cpp_fdiffgrowthl,cld(x),n,diff,fill,by[[1L]],by[[2L]],by[[3L]],G_t(t),return,rho,stubs,power)
 }
 
-DG_pdata_frame_template <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, return = 1L, rho = 1, stubs = TRUE,
+DG_pdata_frame_template <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, return = 1L, rho = 1, stubs = TRUE, shift = "time",
                           keep.ids = TRUE, power = 1, ...) {
 
   if(!missing(...)) unused_arg_action(match.call(), ...)
   ax <- attributes(x)
   nam <- ax[["names"]]
-  index <- unclass(getpix(ax[["index"]]))
+  index <- uncl2pix(x)
+  cols_fun <- is.function(cols)
 
-  if(keep.ids) {
-    gn <- which(nam %in% names(index))
-    if(length(gn) && is.null(cols)) cols <- seq_along(unclass(x))[-gn]
+  if(cols_fun && identical(cols, is.numeric)) cols <- which(.Call(C_vtypes, x, 1L))
+  else if(length(cols)) cols <- cols2int(cols, x, nam, FALSE)
+
+  if(cols_fun || keep.ids) {
+    gn <- which(nam %in% attr(index, "nam")) # Needed for 1 or 3+ index variables
+    if(length(gn)) {
+      if(cols_fun) cols <- fsetdiff(cols, gn)
+      else if(is.null(cols)) cols <- seq_along(unclass(x))[-gn]
+    }
+    if(!keep.ids) gn <- NULL
   } else gn <- NULL
 
-  if(length(index) > 2L) index <- list(finteraction(index[-length(index)]), index[[length(index)]])
   g <- index[[1L]]
-  t <- index[[2L]]
-  tlev <- attr(t, "levels")
-  oldopts <- options(warn = -1L)
-  on.exit(options(oldopts))
-  if(is.finite(as.integer(tlev[1L]))) t <- as.integer(tlev)[t]
+  t <- switch(shift, time = index[[2L]], row = NULL, stop("'shift' must be either 'time' or 'row'"))
+  if(length(t) && !any(ax$class == "indexed_frame")) t <- plm_check_time(t)
 
   cld <- function(y) switch(return, y, fdapply(y, baselog), if(rho == 1) fdapply(y, baselog) else fdapply(y, function(k) rho * baselog(k)), y)
 
-  if(length(cols)) cols <- cols2int(cols, x, nam, FALSE)
-
   if(length(gn) && length(cols)) {
     class(x) <- NULL # Works for multiple lags !
     res <- c(x[gn], .Call(Cpp_fdiffgrowthl,cld(x[cols]),n,diff,fill,fnlevels(g),g,NULL,t,return,rho,stubs,power))
@@ -294,8 +272,8 @@ D.default <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, rho = 1
   fdiff.default(x, n, diff, g, t, fill, FALSE, rho, stubs, ...)
 }
 
-D.pseries <- function(x, n = 1, diff = 1, fill = NA, rho = 1, stubs = TRUE, ...)
-  fdiff.pseries(x, n, diff, fill, FALSE, rho, stubs, ...)
+D.pseries <- function(x, n = 1, diff = 1, fill = NA, rho = 1, stubs = TRUE, shift = "time", ...)
+  fdiff.pseries(x, n, diff, fill, FALSE, rho, stubs, shift, ...)
 
 # setOldClass("pseries")
 # setMethod("D", signature(expr = "pseries"), D.pseries)
@@ -314,11 +292,11 @@ D.data.frame <- function(x, n = 1, diff = 1, by = NULL, t = NULL, cols = is.nume
                          fill = NA, rho = 1, stubs = TRUE, keep.ids = TRUE, ...)
   DG_data_frame_template(x, n, diff, by, t, cols, fill, 1L, rho, stubs, keep.ids, ...)
 
-D.list <- D.data.frame
+D.list <- function(x, ...) D.data.frame(x, ...)
 
-D.pdata.frame <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, rho = 1, stubs = TRUE,
+D.pdata.frame <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, rho = 1, stubs = TRUE, shift = "time",
                           keep.ids = TRUE, ...)
-  DG_pdata_frame_template(x, n, diff, cols, fill, 1L, rho, stubs, keep.ids, ...)
+  DG_pdata_frame_template(x, n, diff, cols, fill, 1L, rho, stubs, shift, keep.ids, ...)
 
 # Log-Difference Operator
 
@@ -329,8 +307,8 @@ Dlog.default <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, rho
   fdiff.default(x, n, diff, g, t, fill, TRUE, rho, stubs, ...)
 }
 
-Dlog.pseries <- function(x, n = 1, diff = 1, fill = NA, rho = 1, stubs = TRUE, ...)
-  fdiff.pseries(x, n, diff, fill, TRUE, rho, stubs, ...)
+Dlog.pseries <- function(x, n = 1, diff = 1, fill = NA, rho = 1, stubs = TRUE, shift = "time", ...)
+  fdiff.pseries(x, n, diff, fill, TRUE, rho, stubs, shift, ...)
 
 Dlog.matrix <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, rho = 1, stubs = TRUE, ...)
   fdiff.matrix(x, n, diff, g, t, fill, TRUE, rho, stubs, ...)
@@ -344,11 +322,11 @@ Dlog.data.frame <- function(x, n = 1, diff = 1, by = NULL, t = NULL, cols = is.n
                          fill = NA, rho = 1, stubs = TRUE, keep.ids = TRUE, ...)
   DG_data_frame_template(x, n, diff, by, t, cols, fill, 2L, rho, stubs, keep.ids, ...)
 
-Dlog.list <- Dlog.data.frame
+Dlog.list <- function(x, ...) Dlog.data.frame(x, ...)
 
-Dlog.pdata.frame <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, rho = 1, stubs = TRUE,
+Dlog.pdata.frame <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, rho = 1, stubs = TRUE, shift = "time",
                           keep.ids = TRUE, ...)
-  DG_pdata_frame_template(x, n, diff, cols, fill, 2L, rho, stubs, keep.ids, ...)
+  DG_pdata_frame_template(x, n, diff, cols, fill, 2L, rho, stubs, shift, keep.ids, ...)
 
 
 # Growth Operator
@@ -360,8 +338,8 @@ G.default <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, logdiff
   fgrowth.default(x, n, diff, g, t, fill, logdiff, scale, power, stubs, ...)
 }
 
-G.pseries <- function(x, n = 1, diff = 1, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, ...)
-  fgrowth.pseries(x, n, diff, fill, logdiff, scale, power, stubs, ...)
+G.pseries <- function(x, n = 1, diff = 1, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, shift = "time", ...)
+  fgrowth.pseries(x, n, diff, fill, logdiff, scale, power, stubs, shift, ...)
 
 G.matrix <- function(x, n = 1, diff = 1, g = NULL, t = NULL, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, ...)
   fgrowth.matrix(x, n, diff, g, t, fill, logdiff, scale, power, stubs, ...)
@@ -375,7 +353,7 @@ G.data.frame <- function(x, n = 1, diff = 1, by = NULL, t = NULL, cols = is.nume
                          fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, keep.ids = TRUE, ...)
   DG_data_frame_template(x, n, diff, by, t, cols, fill, 4L-logdiff, scale, stubs, keep.ids, power, ...)
 
-G.list <- G.data.frame
+G.list <- function(x, ...) G.data.frame(x, ...)
 
-G.pdata.frame <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, keep.ids = TRUE, ...)
-  DG_pdata_frame_template(x, n, diff, cols, fill, 4L-logdiff, scale, stubs, keep.ids, power, ...)
+G.pdata.frame <- function(x, n = 1, diff = 1, cols = is.numeric, fill = NA, logdiff = FALSE, scale = 100, power = 1, stubs = TRUE, shift = "time", keep.ids = TRUE, ...)
+  DG_pdata_frame_template(x, n, diff, cols, fill, 4L-logdiff, scale, stubs, shift, keep.ids, power, ...)
