@@ -14,7 +14,7 @@ demean <- function(x, fl, weights, ..., means = FALSE) {
     # if(!is.matrix(x)) dim(res) <- NULL # also need for flmres... e.g. with weights... intercept is no longer always added, so res needs to be a matrix...
     # Need matrix dimensions... for subset in variable.wise... do.call(cbind, fl[!fc]) needs to be preserved... # return(if(means) x - drop(res) else drop(res))
   if(is.atomic(res)) return(duplAttributes(x - res, x))
-  duplAttributes(mapply(`-`, unattrib(x), unattrib(res), SIMPLIFY = FALSE, USE.NAMES = FALSE), x)
+  duplAttributes(.mapply(`-`, list(unattrib(x), unattrib(res)), NULL), x)
 }
 
 myModFrame <- function(f, data) {
