@@ -5,8 +5,8 @@ fnobs <- function(x, ...) UseMethod("fnobs") # , x
 
 fnobs.default <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, ...) {
   if(is.matrix(x) && !inherits(x, "matrix")) return(fnobs.matrix(x, g, TRA, use.g.names, ...))
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(TRA)) {
+    if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) return(.Call(C_fnobs,x,0L,0L))
     if(is.atomic(g)) {
       if(use.g.names) {
@@ -22,19 +22,14 @@ fnobs.default <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, ...) {
     if(use.g.names) return(`names<-`(.Call(C_fnobs,x,g[[1L]],g[[2L]]), GRPnames(g)))
     return(.Call(C_fnobs,x,g[[1L]],g[[2L]]))
   }
-  if(is.null(g)) return(.Call(Cpp_TRA,x,.Call(C_fnobs,x,0L,0L),0L,TtI(TRA)))
-  if(is.atomic(g)) {
-    if(is.nmfactor(g)) return(.Call(Cpp_TRA,x,.Call(C_fnobs,x,fnlevels(g),g),g,TtI(TRA)))
-    g <- qG(g, na.exclude = FALSE)
-    return(.Call(Cpp_TRA,x,.Call(C_fnobs,x,attr(g,"N.groups"),g),g,TtI(TRA)))
-  }
-  if(!is_GRP(g)) g <- GRP.default(g, return.groups = FALSE, call = FALSE)
-  .Call(Cpp_TRA,x,.Call(C_fnobs,x,g[[1L]],g[[2L]]),g[[2L]],TtI(TRA))
+  if(is.null(g)) return(TRAC(x,.Call(C_fnobs,x,0L,0L),0L,TRA, ...))
+  g <- G_guo(g)
+  TRAC(x,.Call(C_fnobs,x,g[[1L]],g[[2L]]),g[[2L]],TRA, ...)
 }
 
 fnobs.matrix <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, drop = TRUE, ...) {
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(TRA)) {
+    if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) return(.Call(C_fnobsm,x,0L,0L,drop))
     if(is.atomic(g)) {
       if(use.g.names) {
@@ -50,19 +45,14 @@ fnobs.matrix <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, drop = TRU
     if(use.g.names) return(`dimnames<-`(.Call(C_fnobsm,x,g[[1L]],g[[2L]],FALSE), list(GRPnames(g), dimnames(x)[[2L]])))
     return(.Call(C_fnobsm,x,g[[1L]],g[[2L]],FALSE))
   }
-  if(is.null(g)) return(.Call(Cpp_TRAm,x,.Call(C_fnobsm,x,0L,0L,TRUE),0L,TtI(TRA)))
-  if(is.atomic(g)) {
-    if(is.nmfactor(g)) return(.Call(Cpp_TRAm,x,.Call(C_fnobsm,x,fnlevels(g),g,FALSE),g,TtI(TRA)))
-    g <- qG(g, na.exclude = FALSE)
-    return(.Call(Cpp_TRAm,x,.Call(C_fnobsm,x,attr(g,"N.groups"),g,FALSE),g,TtI(TRA)))
-  }
-  if(!is_GRP(g)) g <- GRP.default(g, return.groups = FALSE, call = FALSE)
-  .Call(Cpp_TRAm,x,.Call(C_fnobsm,x,g[[1L]],g[[2L]],FALSE),g[[2L]],TtI(TRA))
+  if(is.null(g)) return(TRAmC(x,.Call(C_fnobsm,x,0L,0L,TRUE),0L,TRA, ...))
+  g <- G_guo(g)
+  TRAmC(x,.Call(C_fnobsm,x,g[[1L]],g[[2L]],FALSE),g[[2L]],TRA, ...)
 }
 
 fnobs.data.frame <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, drop = TRUE, ...) {
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(TRA)) {
+    if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) return(.Call(C_fnobsl,x,0L,0L,drop))
     if(is.atomic(g)) {
       if(use.g.names && !inherits(x, "data.table")) {
@@ -79,21 +69,14 @@ fnobs.data.frame <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, drop =
       return(setRnDF(.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE), groups))
     return(.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE))
   }
-  if(is.null(g)) return(.Call(Cpp_TRAl,x,.Call(C_fnobsl,x,0L,0L,TRUE),0L,TtI(TRA)))
-  if(is.atomic(g)) {
-    if(is.nmfactor(g)) return(.Call(Cpp_TRAl,x,.Call(C_fnobsl,x,fnlevels(g),g,FALSE),g,TtI(TRA)))
-    g <- qG(g, na.exclude = FALSE)
-    return(.Call(Cpp_TRAl,x,.Call(C_fnobsl,x,attr(g,"N.groups"),g,FALSE),g,TtI(TRA)))
-  }
-  if(!is_GRP(g)) g <- GRP.default(g, return.groups = FALSE, call = FALSE)
-  .Call(Cpp_TRAl,x,.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE),g[[2L]],TtI(TRA))
+  if(is.null(g)) return(TRAlC(x,.Call(C_fnobsl,x,0L,0L,TRUE),0L,TRA, ...))
+  g <- G_guo(g)
+  TRAlC(x,.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE),g[[2L]],TRA, ...)
 }
 
-fnobs.list <- function(x, g = NULL, TRA = NULL, use.g.names = TRUE, drop = TRUE, ...)
-  fnobs.data.frame(x, g, TRA, use.g.names, drop, ...)
+fnobs.list <- function(x, ...) fnobs.data.frame(x, ...)
 
 fnobs.grouped_df <- function(x, TRA = NULL, use.g.names = FALSE, keep.group_vars = TRUE, ...) {
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x, call = FALSE)
   nam <- attr(x, "names")
   gn <- which(nam %in% g[[5L]])
@@ -103,6 +86,7 @@ fnobs.grouped_df <- function(x, TRA = NULL, use.g.names = FALSE, keep.group_vars
     ax <- attributes(x)
     attributes(x) <- NULL
     if(nTRAl) {
+      if(!missing(...)) unused_arg_action(match.call(), ...)
       ax[["groups"]] <- NULL
       ax[["class"]] <- fsetdiff(ax[["class"]], c("GRP_df", "grouped_df"))
       ax[["row.names"]] <- if(use.g.names) GRPnames(g) else .set_row_names(g[[1L]])
@@ -119,11 +103,11 @@ fnobs.grouped_df <- function(x, TRA = NULL, use.g.names = FALSE, keep.group_vars
       } else return(setAttributes(.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE), ax))
     } else if(keep.group_vars) {
       ax[["names"]] <- c(nam[gn], nam[-gn])
-      return(setAttributes(c(x[gn],.Call(Cpp_TRAl,x[-gn],.Call(C_fnobsl,x[-gn],g[[1L]],g[[2L]],FALSE),g[[2L]],TtI(TRA))), ax))
+      return(setAttributes(c(x[gn],TRAlC(x[-gn],.Call(C_fnobsl,x[-gn],g[[1L]],g[[2L]],FALSE),g[[2L]],TRA, ...)), ax))
     }
     ax[["names"]] <- nam[-gn]
-    return(setAttributes(.Call(Cpp_TRAl,x[-gn],.Call(C_fnobsl,x[-gn],g[[1L]],g[[2L]],FALSE),g[[2L]],TtI(TRA)), ax))
-  } else return(.Call(Cpp_TRAl,x,.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE),g[[2L]],TtI(TRA)))
+    return(setAttributes(TRAlC(x[-gn],.Call(C_fnobsl,x[-gn],g[[1L]],g[[2L]],FALSE),g[[2L]],TRA, ...), ax))
+  } else return(TRAlC(x,.Call(C_fnobsl,x,g[[1L]],g[[2L]],FALSE),g[[2L]],TRA, ...))
 }
 
 fNobs <- function(x, ...) {
